package widebase.stream.handler.cq

import java.io. { PrintWriter, StringWriter }

import net.liftweb.common.Logger

import org.jboss.netty.channel. {

  ChannelHandlerContext,
  ExceptionEvent,
  MessageEvent,
  SimpleChannelUpstreamHandler

}

import org.jboss.netty.handler.timeout. {

  ReadTimeoutException,
  WriteTimeoutException

}

import widebase.stream.codec. { BadMessage, DoneMessage }

import widebase.stream.codec.cq. {

  FindMessage,
  LoadMessage,
  MessageType,
  SaveMessage,
  TableMessage,
  TableFoundMessage,
  TableNotFoundMessage

}

import widebase.stream.handler.AuthHandler

/** Handles server.
 *
 * @param cache of database
 *
 * @author myst3r10n
 */
class ServerHandler(cache: TableMap)
  extends SimpleChannelUpstreamHandler
  with Logger {

  override def exceptionCaught(ctx: ChannelHandlerContext, evt: ExceptionEvent) {

    evt.getCause match {

      case _: ReadTimeoutException =>
        error("Read timeout")
        evt.getChannel.close.awaitUninterruptibly

      case _: WriteTimeoutException =>
        error("Write timeout")
        evt.getChannel.close.awaitUninterruptibly

      case _ =>
        val message = new StringWriter
        val printer = new PrintWriter(message)
        evt.getCause.printStackTrace(printer)
        error(message.toString)
        evt.getChannel.close

    }
  }

  override def messageReceived(ctx: ChannelHandlerContext, evt: MessageEvent) {

    evt.getMessage match {

      case message: FindMessage =>
        if(!evt.getChannel.getPipeline.get(classOf[AuthHandler])
          .hasAuthorization(evt.getChannel, MessageType.FindMessage.toString))
          return

        if(cache.contains(message.name))
          evt.getChannel.write(new TableFoundMessage)
        else
          evt.getChannel.write(new TableNotFoundMessage)

      case message: LoadMessage =>
        if(!evt.getChannel.getPipeline.get(classOf[AuthHandler])
          .hasAuthorization(evt.getChannel, MessageType.LoadMessage.toString))
          return

        if(cache.contains(message.name)) {

          evt.getChannel.write(new TableMessage(cache(message.name)))

          debug("Loaded table: " + message.name + " by " +
            evt.getChannel.getPipeline.get(classOf[AuthHandler]).username +
            " @ " + evt.getRemoteAddress)

        } else {

          debug("Table not found: " + message.name + " by " +
            evt.getChannel.getPipeline.get(classOf[AuthHandler]).username +
            " @ " + evt.getRemoteAddress)

          evt.getChannel.write(new TableNotFoundMessage)

        }

      case message: SaveMessage =>
        if(!evt.getChannel.getPipeline.get(classOf[AuthHandler])
          .hasAuthorization(evt.getChannel, MessageType.SaveMessage.toString))
          return

        if(cache.contains(message.name))
          cache(message.name) = message.table
        else
          cache += message.name -> message.table

        debug("Saved table: " + message.name + " by " +
          evt.getChannel.getPipeline.get(classOf[AuthHandler]).username
          + " @ " + evt.getRemoteAddress)

        evt.getChannel.write(new DoneMessage)

      case message =>
        error("Unfamiliar with message: " + message)
        evt.getChannel.write(new BadMessage)

    }
  }
}

